@@ -1,16 +1,15 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import unicode_literals
-
-import collections
 import datetime
+from typing import Dict
+from typing import FrozenSet
+from typing import List
+from typing import NamedTuple
+from typing import Tuple
 
 import flask
 
 from git_code_debt.server import logic
-from git_code_debt.server.metric_config import color_overrides
-from git_code_debt.server.metric_config import groups
-from git_code_debt.server.presentation.delta import DeltaPresenter
+from git_code_debt.server.metric_config import Config
+from git_code_debt.server.presentation.delta import Delta
 from git_code_debt.server.render_mako import render_template
 from git_code_debt.util.time import to_timestamp
 
@@ -27,23 +26,15 @@ DATE_NAMES_TO_TIMEDELTAS = (
 )
 
 
-GroupPresenter = collections.namedtuple('GroupPresenter', ['name', 'metrics'])
-
-
-class MetricPresenter(collections.namedtuple(
-        'MetricPresenter',
-        [
-            'name',
-            'color_override',
-            'current_value',
-            'historic_deltas',
-            'all_data_url',
-        ],
-)):
-    __slots__ = ()
+class Metric(NamedTuple):
+    name: str
+    color_override: bool
+    current_value: int
+    historic_deltas: Tuple[Delta, ...]
+    all_data_url: str
 
     @property
-    def classname(self):
+    def classname(self) -> str:
         if self.color_override:
             return 'color-override'
         else:
@@ -52,18 +43,19 @@ class MetricPresenter(collections.namedtuple(
     @classmethod
     def from_data(
             cls,
-            metric_name,
-            today_timestamp,
-            offsets,
-            current_values,
-            metric_data,
-    ):
+            metric_name: str,
+            today_timestamp: int,
+            offsets: List[Tuple[str, int]],
+            current_values: Dict[str, int],
+            metric_data: Dict[str, Dict[str, int]],
+            color_overrides: FrozenSet[str],
+    ) -> 'Metric':
         return cls(
             metric_name,
             metric_name in color_overrides,
             current_values[metric_name],
             tuple(
-                DeltaPresenter(
+                Delta(
                     flask.url_for(
                         'graph.show',
                         metric_name=metric_name,
@@ -81,49 +73,53 @@ class MetricPresenter(collections.namedtuple(
         )
 
 
+class Group(NamedTuple):
+    name: str
+    metrics: Tuple[Metric, ...]
+
+
 def format_groups(
-        metric_names,
-        today_timestamp,
-        offsets,
-        current_values,
-        metric_data,
-):
-    metric_presenters = {
-        metric_name: MetricPresenter.from_data(
+        config: Config,
+        metric_names: List[str],
+        today_timestamp: int,
+        offsets: List[Tuple[str, int]],
+        current_values: Dict[str, int],
+        metric_data: Dict[str, Dict[str, int]],
+) -> List[Group]:
+    metrics = {
+        metric_name: Metric.from_data(
             metric_name,
             today_timestamp,
             offsets,
             current_values,
             metric_data,
+            config.color_overrides,
         )
         for metric_name in metric_names
     }
 
     defined_groups = [
-        GroupPresenter(
+        Group(
             group.name, tuple(
-                metric_presenters[metric_name]
+                metrics[metric_name]
                 for metric_name in metric_names if group.contains(metric_name)
             ),
         )
-        for group in groups
+        for group in config.groups
     ]
 
-    uncategorized_group = GroupPresenter(
+    uncategorized_group = Group(
         'Uncategorized',
         tuple(
-            metric_presenters[metric_name]
+            metrics[metric_name]
             for metric_name in metric_names if not any(
-                group.contains(metric_name) for group in groups
+                group.contains(metric_name) for group in config.groups
             )
         ),
     )
 
-    all_group = GroupPresenter(
-        'All',
-        tuple(
-            metric_presenters[metric_name] for metric_name in metric_names
-        ),
+    all_group = Group(
+        'All', tuple(metrics[metric_name] for metric_name in metric_names),
     )
 
     all_groups = defined_groups + [uncategorized_group, all_group]
@@ -131,8 +127,8 @@ def format_groups(
 
 
 @index.route('/')
-def show():
-    metric_names = logic.get_metric_ids_from_database()
+def show() -> str:
+    metric_names = logic.get_metric_ids(flask.g.db)
     today = datetime.datetime.today()
     today_timestamp = to_timestamp(today)
     offsets = [
@@ -149,6 +145,7 @@ def show():
     return render_template(
         'index.mako',
         groups=format_groups(
+            flask.g.config,
             metric_names,
             today_timestamp,
             offsets,
