@@ -1,16 +1,22 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import itertools
+from typing import Generator
+from typing import Iterable
+from typing import Tuple
+from typing import TypeVar
+
+T = TypeVar('T')
 
 
-def chunk_iter(iterable, n):
+def chunk_iter(
+        iterable: Iterable[T],
+        n: int,
+) -> Generator[Tuple[T, ...], None, None]:
     """Yields an iterator in chunks
 
     For example you can do
 
     for a, b in chunk_iter([1, 2, 3, 4, 5, 6], 2):
-        print('{0} {1}'.format(a, b))
+        print('{} {}'.format(a, b))
 
     # Prints
     # 1 2
